
/* Chrysalide - Outil d'analyse de fichiers binaires
 * plugin-int.h - prototypes pour les structures internes des greffons
 *
 * Copyright (C) 2010-2017 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Chrysalide.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _FORMAT_PLUGINS_PLUGIN_INT_H
#define _FORMAT_PLUGINS_PLUGIN_INT_H


#include <glib-object.h>
#include <stdbool.h>


#include "plugin.h"
#include "plugin-def.h"
#include "../analysis/content.h"
#include "../common/bits.h"
#include "../gui/panels/log.h"



/* Prend acte du [dé]chargement du greffon. */
typedef bool (* pg_management_fc) (GPluginModule *);

/* Exécute une action pendant un désassemblage de binaire. */
typedef void (* pg_process_disassembly_fc) (const GPluginModule *, PluginAction, GLoadedBinary *);




/* Procède à une opération liée à un contenu binaire. */
typedef void (* pg_handle_content) (const GPluginModule *, PluginAction, GBinContent *, wgroup_id_t, GtkStatusStack *);

/* Procède à une opération liée au format de fichier uniquement. */
typedef bool (* pg_handle_format) (const GPluginModule *, PluginAction, GBinFormat *, GtkStatusStack *);

/* Procède à un préchargement de format de fichier. */
typedef bool (* pg_preload_format) (const GPluginModule *, PluginAction, GBinFormat *, GPreloadInfo *, GtkStatusStack *);






/* Précise le nom associé au greffon. */
//typedef char * (* get_plugin_name_fc) (void);

/* Procède à l'initialisation du greffon. */
//typedef bool (* init_plugin_fc) (GPluginModule *, GObject *);

/* Procède à l'extinction du greffon. */
//typedef void (* exit_plugin_fc) (GPluginModule *);

/* Fournit une indication sur le type d'opération(s) menée(s). */
//typedef PluginAction (* get_plugin_action_fc) (const GPluginModule *);

/* Identifie un format à associer à un contenu binaire. */
//typedef MatchingFormatAction (* is_matching_fc) (const GPluginModule *, char **, bin_t **, off_t *);

/* Exécute une action définie sur un binaire chargé. */
//typedef bool (* execute_action_on_binary_fc) (const GPluginModule *, GLoadedBinary *, PluginAction);

/* Exécute une action relative à un débogueur. */
//typedef bool (* execute_on_debugger_fc) (const GPluginModule *, GBinaryDebugger *, PluginAction);


/* Greffon pour Chrysalide (instance) */
struct _GPluginModule
{
    GObject parent;                         /* A laisser en premier        */

    char *filename;                         /* Fichier associé au greffon  */
    GModule *module;                        /* Abstration de manipulation  */

    const plugin_interface *interface;      /* Déclaration d'interfaçage   */

    PluginStatusFlags flags;                /* Fanion pour l'état courant  */

    bitfield_t *dependencies;               /* Cartographie des dépendances*/

    pg_management_fc init;                  /* Procédure d'initialisation  */
    pg_management_fc exit;                  /* Procédure d'extinction      */

    pg_process_disassembly_fc process_disass; /* Catégorie 'désassemblage' */


    //char *name;                             /* Nom associé au greffon      */
    //PluginType type;                        /* Type(s) du greffon          */

    //init_plugin_fc init;                    /* Procédure d'initialisation  */
    //exit_plugin_fc exit;                    /* Procédure d'extinction      */
    //get_plugin_action_fc get_action;        /* Opération(s) menée(s)       */

    //is_matching_fc is_matching;             /* Recherche de correspondance */

    //execute_action_on_binary_fc exec_on_bin;/* Action sur un binaire       */
    //execute_on_debugger_fc handle_debugger; /* Action liée à un débogueur  */


    pg_handle_content handle_content;       /* Explorations ou résolutions */
    pg_handle_format handle_format;         /* Manipulation du format      */
    pg_preload_format preload_format;       /* Préchargement d'un format   */

};


/* Greffon pour Chrysalide (classe) */
struct _GPluginModuleClass
{
    GObjectClass parent;                    /* A laisser en premier        */

};



/* Présente dans le journal un message simple. */
void g_plugin_module_log_simple_message(const GPluginModule *, LogMessageType, const char *);

/* Présente dans le journal un message complexe. */
void g_plugin_module_log_variadic_message(const GPluginModule *, LogMessageType, const char *, ...);








#endif  /* _FORMAT_PLUGINS_PLUGIN_INT_H */
